#!/usr/bin/env lua

local http = require("socket.http")

local string, table = string, table
local format, match, gmatch = string.format, string.match, string.gmatch
local insert = table.insert
local sort = function(t, ...) table.sort(t, ...) return t end

local function read_csv(filename)
  local datafile = io.open(filename, "r")
  local data = {}
  for id, price in gmatch(datafile:read("*all"), "(%d+)%s+(%d+)") do
    data[tonumber(id)] = tonumber(price)
  end
  datafile:close()
  return data
end

local function get_page(url)
  print(format("Fetching page from %s", url))
  local r, e = http.request(url)
  assert(r, e)
  return r
end

local function item_url(id)
  return format("http://www.wowhead.com/?item=%d", id)
end

local function scan_item_urls(page)
  local items, seen = {}, {}
  for idstr in gmatch(page, "?item=(%d+)") do
    if not seen[idstr] then 
      seen[idstr] = true
      insert(items, tonumber(idstr))
    end
  end
  return sort(items)
end

local function scan_coin(str, coinName)
  return tonumber(match(str, format('money%s%%D-(%%d+)', coinName))) or 0
end

local function scan_item_page(page, id)
  local name = match(page, "<title>%s*(.-)%s+-%s+Items%s+-%s+World%sof%sWarcraft</title>")
  if not name or name == "" then print(format("Skipping item:%d: Could not find item name!", id)) return end
  local money_div = match(page, "<%s*div%s*>%s*Sells%s+for%s*(.-)%s*</%s*div%s*>")
  if not money_div or money_div == "" then return name end
  local g, s, c = scan_coin(money_div, "gold"), scan_coin(money_div, "silver"), scan_coin(money_div, "copper")
  return name, g*10000 + s*100 + c
end

local prices = read_csv("data.csv")

local latest_additions_page = get_page("http://www.wowhead.com/?latest=additions")

local new_item_urls = {}
for _, id in ipairs(scan_item_urls(latest_additions_page)) do
  if not prices[id] then
    insert(new_item_urls, id)
  end
end
if #new_item_urls == 0 then print("No new items found") return end
print(format("Found %d new item urls...", #new_item_urls))

for i, id in ipairs(new_item_urls) do
  local page = get_page(item_url(id))
  local name, price = scan_item_page(page, id)
  if name then print(format(" * %q = %s", name, price and tostring(price) or "(no price)")) end
end






